package com.yihu.hos.central.rest.services.errorUpload;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.elasticsearch.ElasticSearchUtil;
import com.yihu.hos.central.rest.constants.RedisCollection;
import com.yihu.hos.central.rest.services.qc.EsSimplePackage;
import com.yihu.hos.core.datatype.DateUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.web.framework.util.springutil.SpringBeanUtil;
import org.apache.commons.collections.CollectionUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.Serializable;
import java.text.ParseException;
import java.util.List;
import java.util.Map;
/**
 *  省平台上传失败处理业务类
 *  1. 将上传失败的数据索引信息保存到ES的upload（upload_fail）中
 *  2. 定时任务读取 重试次数大于3 的ES数据，放入redsi错误重传队列（provincial_platform_queue_fail)
 *  3. 定时任务消费 redsi队列做重传操作，成功则删除es记录(更新效率较低，错误数据暂时无用可删除），失败则ES数据的重传次数+1
 * @author hzy
 * @version 1.0
 * @created 2018.06.12
 */
@Service
public class ErrorUploadService {

    @Autowired
    private ElasticSearchUtil elasticSearchUtil;
    @Autowired
    private ObjectMapper objectMapper;
    private static final String INDEX = "upload";
    private static final String TYPE_UPLOAD_FAIL = "upload_fail";
    @Value("${schemaVersion}")
    private String schemaVersion;


    /**添加上传省平台失败记录至ES */
    @Async
    public void addUploadFailData(EsSimplePackage pack){
        try {
            pack.setUpdate_date(DateUtil.getCurrentString(DateUtil.DEFAULT_YMDHMSDATE_FORMAT));
            List<Map<String, Object>> list = elasticSearchUtil.list(INDEX, TYPE_UPLOAD_FAIL, "rowkey="+ pack.getRowkey()+";retry_times<3");
            if(!CollectionUtils.isEmpty(list)){
                Map<String,Object> oldData = list.get(0);
                int retry_times = Integer.parseInt(oldData.get("retry_times").toString());
                pack.setRetry_times(retry_times+1);
                elasticSearchUtil.voidUpdate(INDEX, TYPE_UPLOAD_FAIL, oldData.get("_id").toString(),objectMapper.readValue(objectMapper.writeValueAsString(pack), Map.class ));
            }else {
                pack.setRetry_times(0);
                pack.setCreate_date(DateUtil.getCurrentString(DateUtil.DEFAULT_YMDHMSDATE_FORMAT));
                elasticSearchUtil.index(INDEX, TYPE_UPLOAD_FAIL, objectMapper.readValue(objectMapper.writeValueAsString(pack), Map.class ));
            }

        } catch (ParseException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /** 抽取ES错误数据到redis重传队列 */
    public void pushData2Queue(){
        try {
            RedisTemplate<String, Serializable> redisTemplate = SpringBeanUtil.getService("redisTemplate");
            Page<Map<String, Object>> list = elasticSearchUtil.page(INDEX, TYPE_UPLOAD_FAIL, "retry_times<3",1,10000);
            for (Map<String, Object> item : list) {
                EsSimplePackage esSimplePackage = new EsSimplePackage();
                esSimplePackage.setRowkey(String.valueOf(item.get("rowkey")));
                esSimplePackage.setPatient_id(String.valueOf(item.get("patient_id")));
                esSimplePackage.setEvent_date(String.valueOf(item.get("event_date")));
                esSimplePackage.setEvent_no(String.valueOf(item.get("event_no")));
                esSimplePackage.setEvent_type( Integer.valueOf(item.get("event_type").toString()));
                esSimplePackage.setOrg_code(String.valueOf(item.get("org_code")));
                esSimplePackage.setIdcard_no(String.valueOf(item.get("demographic_id")));
                esSimplePackage.setPatient_name(String.valueOf(item.get("patient_name")));
                esSimplePackage.setOrg_name(String.valueOf(item.get("org_name")));
                //存入省平台失败重传队列
                redisTemplate.opsForList().leftPush(RedisCollection.PROVINCIAL_PLATFORM_QUEUE_FAIL, objectMapper.writeValueAsString(esSimplePackage));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**删除es中 上传省平台失败记录 */
    public void delUploadFailData(EsSimplePackage pack){
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .must(QueryBuilders.termQuery("rowkey", pack.getRowkey()));
        elasticSearchUtil.deleteByFilter(INDEX, TYPE_UPLOAD_FAIL, queryBuilder);
    }

    /**
     *  通过更细时间删除es数据
     * @param startDate 起始时间
     * @param endDate   结束时间
     */
    public void deleteByUpdateDate(String type,String dateField,String startDate,String endDate) throws  Exception{
        RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery(dateField);
        if (StringUtil.isStrEmpty(endDate)) {
            rangeQuery.gt(startDate);
        }else {
            rangeQuery.from(startDate).to(endDate);
        }
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .must(rangeQuery);
        elasticSearchUtil.deleteByFilter(INDEX, type, queryBuilder);

    }


    public void deleteEsData(String index,String type,String filters) throws  Exception{
        QueryBuilder boolQueryBuilder = elasticSearchUtil.getQueryBuilder(filters);
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .must(boolQueryBuilder);
        elasticSearchUtil.deleteByFilter(index, type, queryBuilder);

    }

}
